#  Copyright (C) 2012 by Carnegie Mellon University.
#  
#  @OPENSOURCE_HEADER_START@
#  Use of the Rayon and related source code is subject to the terms of
#  the following licenses:
#  
#  GNU Public License (GPL) Rights pursuant to Version 2, June 1991
#  Government Purpose License Rights (GPLR) pursuant to DFARS 252.227.7013
#  
#  NO WARRANTY
#  
#  ANY INFORMATION, MATERIALS, SERVICES, INTELLECTUAL PROPERTY OR OTHER 
#  PROPERTY OR RIGHTS GRANTED OR PROVIDED BY CARNEGIE MELLON UNIVERSITY 
#  PURSUANT TO THIS LICENSE (HEREINAFTER THE "DELIVERABLES") ARE ON AN 
#  "AS-IS" BASIS. CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY 
#  KIND, EITHER EXPRESS OR IMPLIED AS TO ANY MATTER INCLUDING, BUT NOT 
#  LIMITED TO, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, 
#  MERCHANTABILITY, INFORMATIONAL CONTENT, NONINFRINGEMENT, OR ERROR-FREE 
#  OPERATION. CARNEGIE MELLON UNIVERSITY SHALL NOT BE LIABLE FOR INDIRECT, 
#  SPECIAL OR CONSEQUENTIAL DAMAGES, SUCH AS LOSS OF PROFITS OR INABILITY 
#  TO USE SAID INTELLECTUAL PROPERTY, UNDER THIS LICENSE, REGARDLESS OF 
#  WHETHER SUCH PARTY WAS AWARE OF THE POSSIBILITY OF SUCH DAMAGES. 
#  LICENSEE AGREES THAT IT WILL NOT MAKE ANY WARRANTY ON BEHALF OF 
#  CARNEGIE MELLON UNIVERSITY, EXPRESS OR IMPLIED, TO ANY PERSON 
#  CONCERNING THE APPLICATION OF OR THE RESULTS TO BE OBTAINED WITH THE 
#  DELIVERABLES UNDER THIS LICENSE.
#  
#  Licensee hereby agrees to defend, indemnify, and hold harmless Carnegie 
#  Mellon University, its trustees, officers, employees, and agents from 
#  all claims or demands made against them (and any related losses, 
#  expenses, or attorney's fees) arising out of, or relating to Licensee's 
#  and/or its sub licensees' negligent use or willful misuse of or 
#  negligent conduct or willful misconduct regarding the Software, 
#  facilities, or other rights or assistance granted by Carnegie Mellon 
#  University under this License, including, but not limited to, any 
#  claims of product liability, personal injury, death, damage to 
#  property, or violation of any laws or regulations.
#  
#  Carnegie Mellon University Software Engineering Institute authored 
#  documents are sponsored by the U.S. Department of Defense under 
#  Contract FA8721-05-C-0003. Carnegie Mellon University retains 
#  copyrights in all material produced under this contract. The U.S. 
#  Government retains a non-exclusive, royalty-free license to publish or 
#  reproduce these documents, or allow others to do so, for U.S. 
#  Government purposes only pursuant to the copyright license under the 
#  contract clause at 252.227.7013.
#  @OPENSOURCE_HEADER_END@
from __future__ import division

from rayon import data, events, toolbox, datatypes

import netsa.data.nice
import netsa.data.format

def get_chart(tools, real_data, metadata):
    # Data
    key_col, val_data, table_data = real_data


    
    # TODO: Someday, we might show more than one column, but not today
    val_col = val_data.column(metadata['show'][0])
    
    
    # Chart
    chart = tools.new_chart('square')

    # Plot
    bar_plot = tools.new_plot('hbar')
    bar_plot.set_data(x=val_col, y=key_col)
    bar_plot.set_scales(bar_width=.7, bar_color=(0, 0, 255, 128),
                        border_color=(0, 0, 0, 255), border_width=1)
    bar_plot.set_metadata(unit_name=metadata['show'][0],
                          # A bit of a hack -- the key will have the
                          # same items in the same order as the
                          # table. (TODO: Maybe just make the key
                          # column in this function?)
                          key_names=table_data.get_column_names())
    chart.add_plot(bar_plot, "bars")

    # Borders
    header_chart = tools.new_chart('square')
    header_tuples = []
    # Bottom border
    def formatfn(x):
        if metadata['show'][0] == 'bytes':
            units = "B"
        else:
            units = ""
        return netsa.data.format.num_prefix(x, units=units)
    # Associate the formatter function in the metadata so the callback
    # can use it.
    bar_plot.set_metadata(formatfn=formatfn)
    xborder = tools.new_border('hnice', line_type=None,
                               scale=bar_plot.get_scale('x'),
                               tick_formatter=formatfn,
                               tickmarker_args=dict(marker_type='none'))
    chart.add_bottom_border(xborder, .05)

    # Left border
    #
    # Use several borders to create a table to the left of the vis
    column_names = table_data.column_names()
    header_child_borders = []
    for idx, col in reversed(list(enumerate(table_data.iter_columns()))):
        # The column
        column_values = [str(c) for c in col]
        tick_tuples = zip(key_col, column_values)
        yscale = bar_plot.get_scale('y')

        try:
            column_name = column_names[idx]
        except IndexError:
            column_name = ""
        
        # If we're doing tick marks on the Y axis, uncomment these
        # lines:
        if idx == (table_data.numcols() - 1):
            marker = tools.new_marker('hline', before=3, after=0)
            border_type = 'v'
        else:
            marker = None
            border_type = 'v'
            
        # If we aren't doing tick marks on the Y axis, uncomment these
        # lines:
        # marker = None
        # border_type = 'v'

        # TODO: It might be worthwhile to make align=0 or 1 and
        # position the "cell" alignment to match the bar width
        # more precisely.
        column_labeler = tools.new_label(halign="right")
        border = tools.new_border(border_type)
        labeledmarker = tools.new_labeled_marker(
            marker=marker, labeler=column_labeler, label_position='w')
        tickset = tools.new_tickset('ranged', tick_tuples=tick_tuples,
                                    scale=yscale, marker=labeledmarker)
        border.add_tickset(tickset)

        def mk_width_function(strings, labeler):
            def width_function(ctx, val):
                bboxes = (labeler.get_bounding_box(ctx, i)[:4]
                          for i in strings)
                
                # TODO: dehacktify ("+ 9"? Seriously??)
                return  max(bbox[3][0] - bbox[0][0] for bbox in bboxes) + 9
                
            return width_function

        width_function = mk_width_function(column_values + [column_name],
                                           column_labeler)
        chart.add_left_border(border, width_function)

        # The header

        # TODO: make this less of a hideous hack.
        header_labeler = tools.new_label(halign='right', valign="bottom")
        
        header_labeledmarker = tools.new_labeled_marker(
            marker=None,
            label_position='w', # (Yes, "w". This way the label
                                # spacing mirrors that of the
                                # columns.)
            labeler=header_labeler)
        header_border = tools.new_border('vtick',
                                         scale=lambda _: 0,
                                         tick_tuples=((0, column_name),),
                                         tickmarker=header_labeledmarker,
                                         line_type=None)
        header_chart.add_left_border(header_border, width_function)
        header_tuples.append((column_name, header_labeler))

    def mk_height_function(tuples):
        def height_function(ctx, val):
            bboxes = (labeler.get_bounding_box(ctx, i)[:4]
                      for i, labeler in tuples)
            return  max(bbox[3][1] - bbox[0][1] for bbox in bboxes)
        return height_function
    # This border gives the corner enough height to draw the
    # headers. TODO: The real, non-hackish way to do this is to
    # ask the corners how high they should be.
    chart.add_top_border(tools.new_border('none'),
                         height=mk_height_function(header_tuples))
        

                            
    chart.add_nw_corner(header_chart)
        
            
        

    # Background color and gridlines
    chart.set_chart_background("#fff")
    chart.set_plot_background("#ddd")

    x_grid = tools.new_gridlines('vertical')
    # Use the outside ticks, minus the first and last
    grid_ticks = tuple(xborder.iter_tick_positions(0))[1:-1]
    x_grid.set_data(pos=data.Column(grid_ticks))
    x_grid.set_scales(pos=bar_plot.get_scale('x'),
                      line_color="white", line_style="solid",
                      line_width=2)
    chart.add_rear_gridlines(x_grid)


            
    # Title
    if metadata.has_key('title'):
        title = tools.new_border('hlabel', label=metadata['title'],
                                 font_size='large')
        chart.add_top_title(title, .1)

    # Padding
    header_chart.set_padding(bpad="4px")
    chart.set_padding(lpad="10px", rpad="20px", tpad="10px", bpad="10px")
    return chart
            
            
    

class MyBarListener(events.EventListener):
    def __init__(self, component, viewpage):
        events.EventListener.__init__(self, component)
        self.viewpage = viewpage

    def _drill_down(self, cat_names, unit_name, cat_data, val_data):
        def wash(d):
            if isinstance(d, datatypes.IPv4):
                return str(d)
            return d
        washed_cat_data = tuple(wash(c) for c in cat_data)
        cat_dict = dict(zip(cat_names, washed_cat_data))
        val_dict = {unit_name: val_data}
        return cat_dict, val_dict
                        
    def on_tooltip_(self, target, evt):
        bars = target.bars_at(evt.x(), evt.y())
        if len(bars) > 0:
            bar = bars[0]
            formatfn = bar.extra_data['formatfn']
            evt.set_tooltip(formatfn(bar.xdata), 1)
            
    def on_mouse_left_dclick_(self, target, evt):
        bars = target.bars_at(evt.x(), evt.y())
        if len(bars) > 0 and self.viewpage is not None:
            bar = bars[0]
            my_plot = target
            cat_dict, val_dict = self._drill_down(
                my_plot.get_metadata()['key_names'],
                my_plot.get_metadata()['unit_name'],
                bar.ydata, bar.xdata)
            # Example of input to RefinementCallback:
            # ({'sip': "192.168.1.1", 'dip': "192.168.1.2", 'dport': 100},
            #  {'bytes': 65535})
            self.viewpage.RefinementCallback((cat_dict, val_dict))
            


            
def set_events(chart, event_source):
    viewpage = event_source.viewpage
    adapter = chart.get_event_source(event_source, activate=False)
    component = chart.get_plot('bars')
    listener = MyBarListener(component, viewpage)
    listener.register_with(adapter)


__all__ = ["get_data", "set_events"]

    
    
