import unittest
from rayon.data import Column, Dataset, RayonDataException
from rayon.miscutils import dedent

from StringIO import StringIO

class t_Row_1(unittest.TestCase):
    colnames = ['foo', 'bar', 'baz', 'quux']
    d1 = Dataset.from_rows(
        [[1, "a", "z", "alpha"],
         [2, "b", "y", "beta"],
         [3, "c", "x", "gamma"]],
        colnames=colnames)
    r = d1.get_row(0)
                
    
    def test_access_1(self):
        self.assertEqual(self.r[0], 1)
    def test_access_2(self):
        self.assertEqual(self.r.foo, 1)
    def test_access_3(self):
        self.assertEqual(self.r[(0,1)], (1, "a"))
    def test_access_4(self):
        self.assertEqual(self.r[('foo', 'bar')], (1, "a"))
    def test_access_5(self):
        self.assertEqual(self.r[(0, 'bar')], (1, "a"))
    def test_get_column_names_1(self):
        self.assertEqual(self.r.get_column_names(), self.colnames)
    def test_get_column_names_2(self):
        self.assertEqual(self.r.get_column_names(synthesize_indices=False), 
                         self.colnames)
    def test_get_column_names_3(self):
        self.assertEqual(self.r.get_column_names(synthesize_indices=True), 
                         self.colnames)
    def test__fields_1(self):
        self.assertEqual(self.r._fields(), self.colnames)

        
                         
        

class t_Row_2(unittest.TestCase):
    # No column names
    d1 = Dataset.from_rows(
        [[1, "a", "z", "alpha"],
         [2, "b", "y", "beta"],
         [3, "c", "x", "gamma"]])
    r = d1.get_row(0)
    def test_get_column_names_1(self):
        self.assertEqual(self.r.get_column_names(), [])
    def test_get_column_names_2(self):
        self.assertEqual(self.r.get_column_names(synthesize_indices=False), [])
    def test_get_column_names_3(self):
        for i, v in zip(self.r.get_column_names(synthesize_indices=True),
                        [1, "a", "z", "alpha"]):
            self.assertEqual(self.r[i], v)
    def test__fields_1(self):
        for i, v in zip(self.r._fields(),
                        [1, "a", "z", "alpha"]):
            self.assertEqual(self.r[i], v)



class t_Dataset___init__(unittest.TestCase):
    def test_1(self):
        c1 = Column([1,2,3,4])
        c2 = Column([5,4,3,2])
        d = Dataset([c1, c2])
        self.assertEquals(c1, d.get_column(0))
        self.assertEquals(c2, d.get_column(1))

    def test_2(self):
        c1 = Column([1,2,3,4])
        c2 = Column([5,4,3,2])
        d = Dataset((c for c in (c1, c2)))
        self.assertEquals(c1, d.get_column(0))
        self.assertEquals(c2, d.get_column(1))

class t_Dataset_from_stream(unittest.TestCase):
    class _(object):
        @staticmethod
        def passes(in_str,
                   expected_cols=None,
                   expected_headers=None,
                   expected_colnames=None,
                   expected_num_cols=None,
                   expected_num_rows=None,
                   debug=False):
            def test_(self):
                if debug:
                    import pdb;
                    pdb.set_trace()
                d = Dataset.from_stream(StringIO(dedent(in_str)))
                if expected_cols is not None:
                    for e_name, e_col in expected_cols:
                        self.assertEqual(e_col, tuple(d.get_column(e_name)))
                if expected_headers is not None:
                    self.assertEqual(expected_headers,
                                     dict(d.get_headers()))
                if expected_colnames is not None:
                    self.assertEqual(expected_colnames,
                                     tuple(d.get_column_names()))
                if expected_num_cols is not None:
                    self.assertEqual(expected_num_cols, d.get_num_columns())
                if expected_num_rows is not None:
                    self.assertEqual(expected_num_rows, d.get_num_rows())
            return test_

        @staticmethod
        def fails(in_str, exc):
            def test_(self):
                self.assertRaises(
                    exc,
                    lambda: Dataset.from_stream(StringIO(dedent(in_str))))
            return test_


    expected_output_1 = (('foo' , (1, 2, 3)),
                         ('bar' , ('a', 'b', 'c')),
                         ('baz' , ('z', 'y', 'x')),
                         ('quux', ('alpha', 'beta', 'gamma')))
   

    test_from_stream_1 = _.passes(
        """|#foo|bar|baz|quux
           |1|a|z|alpha
           |2|b|y|beta
           |3|c|x|gamma""",
        expected_cols=expected_output_1)
    
    
    test_whitespace_1 = _.passes(
        """|# foo | bar | baz | quux
           |    1 | a   | z   | alpha
           |    2 | b   | y   | beta
           |    3 | c   | x   | gamma""",
        expected_cols=expected_output_1)

        
    test_new_column_names_1 = _.passes(
        """|## Column-names: foo|bar|baz|quux
           |1|a|z|alpha
           |2|b|y|beta
           |3|c|x|gamma""",
        expected_cols=expected_output_1)
    
    test_delimiter_1 = _.passes(
        """|## Delimiter:,
           |## Column-names: foo,bar,baz,quux
           |1,a,z,alpha
           |2,b,y,beta
           |3,c,x,gamma""",
        expected_cols=expected_output_1)    
    
    test_just_data_1 = _.passes(
        """|a|b|c|d|e""",
        expected_cols = ((0, ('a',)),
                         (1, ('b',)),
                         (2, ('c',)),
                         (3, ('d',)),
                         (4, ('e',))),
        debug=False)
    
    test_just_data_2 = _.passes(
        """|a|b|c|d|e
           |f|g|h|i|j""",
        expected_cols = ((0, ('a', 'f')),
                         (1, ('b', 'g')),
                         (2, ('c', 'h')),
                         (3, ('d', 'i')),
                         (4, ('e', 'j'))),
        debug=False)
    

    test_no_data_1 = _.passes(
        """|#foo|bar|baz|quux""",
        expected_num_cols=4,
        expected_num_rows=0)


    test_blank_lines_1 = _.passes(
        """|#a|b|c
           |
           | A|1|z
           | B|2|y
           |
           | C|3|x""",
        expected_cols=(('a', ('A', 'B', 'C')),
                       ('b', (1, 2, 3)),
                       ('c', ('z', 'y', 'x'))))

    test_comments_ignored_1 = _.passes(
        """|#a|b|c
           | A|1|z
           | B|2|y
           |# A random comment
           | C|3|x""",
        expected_cols=(('a', ('A', 'B', 'C')),
                       ('b', (1, 2, 3)),
                       ('c', ('z', 'y', 'x'))))

    # Missing cells have unusual behavior, based on whether they occur
    # in the first or subsequent rows. If no typemap is supplied, the
    # first row is used to infer the column types, so if data is
    # missing from there, the type will be inferred as str. If not, it
    # may get inferred as a numeric type, which can't handle missing
    # data, and reading the dataset will fail.
    #
    # The best thing to do if you expect empty cells is to specify
    # typemaps yourself that can handle missing data.
    test_missing_cells_1 = _.passes(
        """|#a|b|c
           |1||3
           |4|5|6""",
        expected_cols = (('a', (1, 4)),
                         ('b', ('', '5')), # str is the inferred type
                         ('c', (3, 6))))

    test_missing_cells_2 = _.fails(
        """|#a|b|c
           |1|2|3
           |4||6""",
        RayonDataException)


class t_Dataset_get_column(unittest.TestCase):
    def test_by_index_1(self):
        d1 = Dataset.from_columns([[1, 2, 3], 
                                   [6, 5, 4], 
                                   [7, 9, 8]])
        self.assertEqual([1, 2, 3], list(d1.get_column(0)))

    def test_by_name_1(self):
        d1 = Dataset.from_columns([[1, 2, 3], 
                                   [6, 5, 4], 
                                   [7, 9, 8]], 
                                  colnames=['foo', 'bar', 'baz'])
        self.assertEqual([1, 2, 3], list(d1.get_column('foo')))

    def test_sorted_1(self):
        d1 = Dataset.from_columns([[1, 2, 3], 
                                   [6, 5, 4], 
                                   [7, 9, 8]])
        d1.sort(key=lambda r: r[1])
        self.assertEqual([3, 2, 1], list(d1.get_column(0)))
        
        
        
def to_tuples(d):
    return tuple(tuple(r) for r in d)

class t_Dataset_get_column_index_from_name(unittest.TestCase):
    def test_by_name_1(self):
        d = Dataset.from_string(dedent("""|#foo|bar
                                          |1|3"""))
        self.assertEqual(0, d.get_column_index_from_name('foo'))

    def test_by_number_1(self):
        d = Dataset.from_string(dedent("""|#foo|bar
                                          |1|3"""))
        self.assertEqual(0, d.get_column_index_from_name(0))

    def test_by_number_2(self):
        d = Dataset.from_string(dedent("""|1|3"""))
        self.assertEqual(0, d.get_column_index_from_name(0))

    def test_by_number_3(self):
        d = Dataset.from_string(dedent("""|#foo|bar
                                          |1|3"""))
        self.assertEqual(1, d.get_column_index_from_name(-1))

    def test_by_number_4(self):
        d = Dataset.from_string(dedent("""|1|3"""))
        self.assertEqual(1, d.get_column_index_from_name(-1))

        
    def test_invalid_name_1(self):
        d = Dataset.from_string(dedent("""|#foo|bar
                                          |1|3"""))
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name('baz'))
        
    def test_invalid_name_2(self):
        d = Dataset.from_string(dedent("""|1|3"""))
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name('anything'))

        
    def test_oob_number_1(self):
        d = Dataset.from_string(dedent("""|#foo|bar
                                          |1|3"""))
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name(2))

    def test_oob_number_2(self):
        d = Dataset.from_string(dedent("""|#foo|bar
                                          |1|3"""))
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name(-3))
        
    def test_empty_1(self):
        d = Dataset()
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name("astring"))
        
    def test_empty_2(self):
        d = Dataset()
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name(0))
        
    def test_empty_3(self):
        d = Dataset()
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name(1))
        
    def test_empty_4(self):
        d = Dataset()
        self.assertRaises(
            RayonDataException,
            lambda: d.get_column_index_from_name(-1))
        
        
class t_Dataset_partition(unittest.TestCase):
    d1 = Dataset.from_rows(
        [["tall", "blue",  10],
         ["short", "blue", 3],
         ["tall", "red",   20],
         ["short", "red",   6],
         ["tall", "green",  30],
         ["short", "green", 9]],
        colnames=("height", "color" ,"volume"),
        header={'foo': 1, 'bar': 2})
    d2 = Dataset.from_rows(
        [["tall", "blue", "squamous", 10],
         ["tall", "blue", "rugose", 20],
         ["tall", "red", "squamous", 30],
         ["tall", "red", "rugose", 40],
         ["tall", "green", "squamous", 50],
         ["tall", "green", "rugose", 60],
         ["short", "blue", "squamous", 3],
         ["short", "blue", "rugose", 6],
         ["short", "red", "squamous", 9],
         ["short", "red", "rugose", 12],
         ["short", "green", "squamous", 15],
         ["short", "green", "rugose", 18]])
    expected_id_1 = (("tall", (("tall", "blue",  10),
                               ("tall", "red", 20),
                               ("tall", "green",   30))),
                     ("short", (("short", "blue", 3),
                                ("short", "red", 6),
                                ("short", "green",  9)))) 
    def test_partition_by_id_1(self):
        new_ds = self.d1.partition(0)
        for ((e_key, e_data), (a_key, a_data)) in zip(self.expected_id_1,
                                                      new_ds):
            self.assertEquals(e_key, a_key)
            self.assertEquals(e_data, to_tuples(a_data))

    def test_partition_by_id_2(self):
        # If the input is a one-element tuple, return a one-element
        # tuple.
        new_ds = self.d1.partition((0,))
        for ((e_key, e_data), (a_key, a_data)) in zip(self.expected_id_1,
                                                      new_ds):
            self.assertEquals((e_key,), a_key)
            self.assertEquals(e_data, to_tuples(a_data))


    def test_partition_by_name_1(self):
        expected = (("tall", (("tall", "blue",  10),
                              ("tall", "red", 20),
                              ("tall", "green",   30))),
                    ("short", (("short", "blue", 3),
                               ("short", "red", 6),
                               ("short", "green",  9))))
        new_ds = self.d1.partition("height")
        for ((e_key, e_data), (a_key, a_data)) in zip(expected, new_ds):
            self.assertEquals(e_key, a_key)
            self.assertEquals(e_data, to_tuples(a_data))
            
    def test_partition_by_name_2(self):
        # If the input is a one-element tuple, return a one-element tuple.
        expected = (("tall", (("tall", "blue",  10),
                              ("tall", "red", 20),
                              ("tall", "green",   30))),
                    ("short", (("short", "blue", 3),
                               ("short", "red", 6),
                               ("short", "green",  9))))
        new_ds = self.d1.partition(("height",))
        for ((e_key, e_data), (a_key, a_data)) in zip(expected, new_ds):
            self.assertEquals((e_key,), a_key)
            self.assertEquals(e_data, to_tuples(a_data))

    def test_partition_by_empty_1(self):
        # Partitioning on empty input
        out = self.d1.partition(tuple())
        self.assertEqual(1, len(out))
        self.assertEqual(tuple(), out[0][0])
        self.assertEqual(self.d1.to_string(), out[0][1].to_string())
            
    def test_copy_header_1(self):
        for name, dset in self.d1.partition(0, copy_header=True):
            headers = tuple(sorted(dset.get_headers()))
            self.assertEqual(headers, (("bar", 2), ("foo", 1)))
    def test_copy_colnames_1(self):
        for name, dset in self.d1.partition(0, copy_colnames=True):
            colnames = tuple(dset.get_column_names())
            self.assertEqual(colnames, ("height", "color", "volume"))
    # copy_sort_key isn't tested because it's deprecated. 
    def test_insert_sorted_1(self):
        d = Dataset.from_rows(
            [["tall", "blue",  10],
             ["short", "blue", 3],
             ["tall", "red",   20],
             ["short", "red",   6],
             ["tall", "green",  30],
             ["short", "green", 9]])
        d.sort(lambda r: (r[0], r[1]))
        expected = {"short": (("short", "blue" , 3),
                              ("short", "green", 9),
                              ("short", "red"  , 6)),
                    "tall" : (("tall" , "blue" , 10),
                              ("tall" , "green", 30),
                              ("tall" , "red"  , 20))}

        new_ds = d.partition(0, insert_sorted=True)
        # Convert new_ds to dict, because order of keys doesn't
        # matter. Convert datasets to tuples, because we only care
        # about order.
        self.assertEqual(expected,
                         dict((k, to_tuples(v)) for k, v in new_ds))
    def test_compound_key_1(self):
        expected = ((("tall", "blue"), (("tall", "blue", "squamous", 10),
                                        ("tall", "blue", "rugose", 20))),
                    (("tall", "red"), (("tall", "red", "squamous", 30),
                                       ("tall", "red", "rugose", 40))),
                    (("tall", "green"), (("tall", "green", "squamous", 50),
                                         ("tall", "green", "rugose", 60))),
                    (("short", "blue"), (("short", "blue", "squamous", 3),
                                         ("short", "blue", "rugose", 6))),
                    (("short", "red"), (("short", "red", "squamous", 9),
                                        ("short", "red", "rugose", 12))),
                    (("short", "green"), (("short", "green", "squamous", 15),
                                          ("short", "green", "rugose", 18))))
        new_ds = self.d2.partition((0, 1))
        for ((e_key, e_data), (a_key, a_data)) in zip(expected, new_ds):
            self.assertEqual(e_key, a_key)
            self.assertEqual(e_data, to_tuples(a_data))
    def test_compound_key_2(self):
        expected = ((('blue', 'tall'), (('tall', 'blue', 'squamous', 10),
                                        ('tall', 'blue', 'rugose', 20))),
                    (('red', 'tall'), (('tall', 'red', 'squamous', 30),
                                       ('tall', 'red', 'rugose', 40))),
                    (('green', 'tall'), (('tall', 'green', 'squamous', 50),
                                         ('tall', 'green', 'rugose', 60))),
                    (('blue', 'short'), (('short', 'blue', 'squamous', 3),
                                         ('short', 'blue', 'rugose', 6))),
                    (('red', 'short'), (('short', 'red', 'squamous', 9),
                                        ('short', 'red', 'rugose', 12))),
                    (('green', 'short'), (('short', 'green', 'squamous', 15),
                                          ('short', 'green', 'rugose', 18))))
        new_ds = self.d2.partition((1, 0))
        for ((e_key, e_data), (a_key, a_data)) in zip(expected, new_ds):
            self.assertEqual(e_key, a_key)
            self.assertEqual(e_data, to_tuples(a_data)) 

class t_Dataset_meld(unittest.TestCase):
    d = Dataset.from_rows(
        [['a', 1, 10],
         ['a', 2, 20],
         ['b', 3, 30]])
    def test_1(self):
        comp = self.d.meld(0, 1)
        self.assertEqual(tuple(comp), (('a', 1), ('a', 2), ('b', 3)))

    def test_single_values_1(self):
        comp = self.d.meld(1)
        self.assertEqual(tuple(comp), (1,2,3))

    def test_no_values_1(self):
        comp = self.d.meld()
        self.assertEqual(tuple(comp), (('a', 1, 10),
                                       ('a', 2, 20),
                                       ('b', 3, 30)))
        
    def test_bad_values_1(self):
        self.assertRaises(RayonDataException,
                          lambda: self.d.meld('foo', 'bar'))
        
    def test_bad_values_2(self):
        self.assertRaises(RayonDataException,
                          lambda: self.d.meld(0, 'bar'))
        
    def test_bad_values_3(self):
        self.assertRaises(RayonDataException,
                          lambda: self.d.meld(0,1,2,3))

class t_Dataset_summarize(unittest.TestCase):
    class _(object):
        @staticmethod
        def passes_1(str_in, str_expected, **kargs):
            d_in = Dataset.from_string(dedent(str_in))
            d_expected = Dataset.from_string(dedent(str_expected))
            def test_(self):
                self.assertEqual(
                    d_expected.to_string(),
                    d_in.summarize(**kargs).to_string())
            return test_

    # Basic functionality (no column names)

    d_1_in = """|a| 1| 10| red  | 30
                |a| 2| 10| green| 20
                |a| 1| 10| red  | 40
                |b| 4| 10| red  | 50
                |b| 0| 10| red  | 10
                |b| 1| 10| red  | 90
                |b| 1| 10| red  | 10
                |c| 4| 10| red  | 30
                |c| 2| 10| blue | 60"""

    d_1_out = """|a|1|10|red  |70
                 |a|2|10|green|20
                 |b|4|10|red  |50
                 |b|0|10|red  |10
                 |b|1|10|red  |100
                 |c|4|10|red  |30
                 |c|2|10|blue |60"""
            
    test_1 = _.passes_1(d_1_in, d_1_out, colname_or_idx=4)

    # Basic functionality (with column names)

    d_colnames_in = """|#foo|bar|baz|quux|yara
                       |a| 1| 10| red  | 30
                       |a| 2| 10| green| 20
                       |a| 1| 10| red  | 40
                       |b| 4| 10| red  | 50
                       |b| 0| 10| red  | 10
                       |b| 1| 10| red  | 90
                       |b| 1| 10| red  | 10
                       |c| 4| 10| red  | 30
                       |c| 2| 10| blue | 60"""

    d_colnames_out = """|#foo|bar|baz|quux|yara
                        |a|1|10|red  |70
                        |a|2|10|green|20
                        |b|4|10|red  |50
                        |b|0|10|red  |10
                        |b|1|10|red  |100
                        |c|4|10|red  |30
                        |c|2|10|blue |60"""

    test_colnames_1 = _.passes_1(
        d_colnames_in, d_colnames_out, colname_or_idx="yara")


    # Arguments are returned in original order

    d_nonlast_col_in = """|#foo|bar|baz|quux
                          |a| 1| 10| red  
                          |a| 2| 10| green
                          |a| 1| 10| red  
                          |b| 4| 10| red  
                          |b| 0| 10| red  
                          |b| 1| 10| red  
                          |b| 1| 10| red  
                          |c| 4| 10| red  
                          |c| 2| 10| blue """

    d_nonlast_col_out = """|#foo|bar|baz|quux
                           |a|1|20|red
                           |a|2|10|green
                           |b|4|10|red
                           |b|0|10|red
                           |b|1|20|red
                           |c|4|10|red
                           |c|2|10|blue """


    test_nonlast_col_1 = _.passes_1(
        d_nonlast_col_in, d_nonlast_col_out, colname_or_idx="baz")

    # Single-element key

    d_keylen1_in = """|## column-names: time|bytes
                      |0|10
                      |1|20
                      |2|30
                      |0|10
                      |1|20
                      |4|10"""

    d_keylen1_out = """|## column-names: time|bytes
                       |0|20
                       |1|40
                       |2|30
                       |4|10"""

    test_nonlast_col_1 = _.passes_1(
        d_keylen1_in, d_keylen1_out, colname_or_idx="bytes")

    # Single-element dataset
    d_numcols1_in =  """|1
                        |2
                        |3"""

    test_numcols1_out = _.passes_1(
        d_numcols1_in, "|6", colname_or_idx=0)


    # Empty dataset
    test_empty_1 = _.passes_1("", "", colname_or_idx="")
    test_empty_2 = _.passes_1("", "", colname_or_idx=0)
    test_empty_3 = _.passes_1("", "", colname_or_idx="wrong")
    test_empty_4 = _.passes_1("|#foo|bar", "|#foo|bar", colname_or_idx="foo")


    


        
if __name__ == '__main__':
    unittest.main()
