import unittest
from rayon.markers import Arrow, LeftArrow, UpArrow, DownArrow, RightArrow
from rayon.miscutils import parse_color

try:
    from nose.tools import set_trace
except ImportError:
    def set_trace():
        import pdb
        pdb.set_trace()


class MyFakeContext(object):
    def draw_lines(self,
                   segments,
                   color,
                   width,
                   fixup=False,
                   dashes=None,
                   dash_offset=0):
        self._segments = tuple(segments)
        self._color=color
        self._width=width
    def test_get_segments(self):
        return self._segments
    def test_get_line_config(self):
        return (self._color, self._width)
                     
        

class t_Arrow(unittest.TestCase):
    def setUp(self):
        self.ctx = MyFakeContext()
    def check_line_config(self, in_color, in_width):
        out_color, out_width = self.ctx.test_get_line_config()
        self.assertEquals(parse_color(in_color),
                          parse_color(out_color))
        self.assertEquals(in_width, out_width)
    
    def check_segments(self, in_seg_1, in_seg_2):
        out_seg_1, out_seg_2 = self.ctx.test_get_segments()
        def check_segment(s1, s2):
            try:
                for i1, i2 in zip(s1, s2):
                    self.assertAlmostEqual(i1, i2, places=4)
            except AssertionError:
                raise AssertionError("%s !~ %s" % (s1, s2))
                
        check_segment(in_seg_1, out_seg_1)
        check_segment(in_seg_2, out_seg_2)
        
    def test_draw_1(self):
        arrow = Arrow()
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (-3.53553, -3.53553, 0, 0), # r=5, theta=225
            (0, 0, 3.53553, -3.53553))  # r=5, theta=315

    def test_draw_2(self):
        arrow = Arrow()
        arrow.draw(self.ctx, 3, 4)
        self.check_line_config("black", 1)
        self.check_segments(
            (-0.535533, 0.464467, 3, 4), # r=5, theta=225
            (3, 4, 6.53553, 0.464467))  # r=5, theta=315

    def test_draw_3(self):
        arrow = Arrow(rotation=10)
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (-4.09576, -2.86788, 0, 0), # r=5, theta=215
            (0, 0, 2.86788, -4.09576))  # r=5, theta=305
             
    def test_draw_4(self):
        arrow = Arrow(angle=40)
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (-1.71010, -4.69846, 0, 0), # r=5, theta=250
            (0, 0, 1.71010, -4.69846))  # r=5, theta=290

    def test_draw_5(self):
        arrow = Arrow(rotation=45)
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (-5, 0, 0, 0), # r=5, theta=250
            (0, 0, 0, -5))  # r=5, theta=290

    def test_draw_6(self):
        arrow = Arrow(size=7)
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (-4.94974, -4.94974, 0, 0), # r=7, theta=225
            (0, 0, 4.94974, -4.94974))  # r=7, theta=315

    def test_draw_7(self):
        arrow = Arrow(rotation=35, angle=17, size=2)
        arrow.draw(self.ctx, 6.1, 9.2)
        self.check_line_config("black", 1)
        self.check_segments(
            (4.72330, 7.74926, 6.1, 9.2),  # r=2, theta=226.5
            (6.1, 9.2, 5.207605, 7.41014)) # r=2, theta=243.5


    def test_draw_8(self):
        arrow = Arrow(stroke_width=3)
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 3)
        self.check_segments(
            (-3.53553, -3.53553, 0, 0), # r=5, theta=225
            (0, 0, 3.53553, -3.53553))  # r=5, theta=315

    def test_draw_9(self):
        arrow = Arrow(stroke_color="#ff0000ff")
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("red", 1)
        self.check_segments(
            (-3.53553, -3.53553, 0, 0), # r=5, theta=225
            (0, 0, 3.53553, -3.53553))  # r=5, theta=315


    # Down Arrow
    def test_draw_down_1(self):
        arrow = UpArrow()
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (3.53553, 3.53553, 0, 0), # r=5, theta=225
            (0, 0, -3.53553, 3.53553))  # r=5, theta=45

    # Left Arrow
    def test_draw_left_1(self):
        arrow = LeftArrow()
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (3.53553, -3.53553, 0, 0), # r=5, theta=315
            (0, 0, 3.53553, 3.53553))  # r=5, theta=45
        
    # Right Arrow
    def test_draw_right_1(self):
        arrow = RightArrow()
        arrow.draw(self.ctx, 0, 0)
        self.check_line_config("black", 1)
        self.check_segments(
            (-3.53553, 3.53553, 0, 0), # r=5, theta=315
            (0, 0, -3.53553, -3.53553))  # r=5, theta=45
        
        
        
        
if __name__ == '__main__':
    unittest.main()


__all__ = """
ArrowMarker
"""
