import unittest
from rayon.plots import HorizontalBarPlot
from rayon import RayonException

class FakeContext(object):
    def __init__(self, interactive=False):
        self._rects = []
        self.interactive = interactive
    def draw_rectangle(self, x, y, width, height,
                       stroke_width, stroke_color, fill_color,
                       fixup=False):
        self._rects.append(dict(
                x=x, y=y, width=width, height=height,
                stroke_width=stroke_width, stroke_color=stroke_color,
                fill_color=fill_color))
    def test_get_rects(self):
        return list(self._rects)

class Rect(object):
    def __init__(self, x, y, width, height):
        self.x = x
        self.y = y
        self.width = width
        self.height = height

class t_HorizontalBarPlot(unittest.TestCase):
    class _(object):
        @staticmethod
        def x_scale_fn(in_val):
            if in_val == 0 : return 0.0
            if in_val == 2 : return 0.2
            if in_val == 5 : return 0.5
            if in_val == 10: return 1.0
            raise KeyError(in_val)

        @staticmethod
        def y_scale_fn(in_val):
            if in_val == 1: return (0.0, 0.5)
            if in_val == 2: return (0.5, 1.0)
            raise KeyError(in_val)

        @staticmethod
        def passes_1(x_origin, x_scale, y_scale, expected_rect):
            def test_(self):
                ctx = FakeContext()
                plot = HorizontalBarPlot(
                    x_origin=x_origin,
                    x_data=(5, 10),
                    y_data=(1, 2), 
                    x_scale=x_scale, 
                    y_scale=y_scale,
                    bar_color_scale=(255, 0, 0, 0),
                    bar_alignment_scale=.5,
                    bar_width_scale=1, border_width_scale=0,
                    bar_fixup_scale=False)

                plot.draw(ctx, 10, 10)
                
                rect = ctx.test_get_rects()[0]
                er = expected_rect
                self.assertEquals(
                    dict(
                        x=er.x, y=er.y,
                        width=er.width, height=er.height,
                        stroke_width=0,
                        stroke_color=(0, 0, 0, 255),
                        fill_color=(255, 0, 0, 0)),
                    rect)
            return test_

        @staticmethod
        def fails_1(x_origin, x_scale, y_scale, exc):
            def test_(self):
                ctx = FakeContext()
                plot = HorizontalBarPlot(
                    x_origin=x_origin,
                    x_data=(5, 10),
                    y_data=(1, 2), 
                    x_scale=x_scale, 
                    y_scale=y_scale,
                    bar_color_scale=(255, 0, 0, 0),
                    bar_alignment_scale=.5,
                    bar_width_scale=1, border_width_scale=0,
                    bar_fixup_scale=False)

                self.assertRaises(exc, lambda: plot.draw(ctx, 10, 10))
            return test_
                

    # No x_origin: use zero
    test_draw_1 = _.passes_1(
        None,
        _.x_scale_fn,
        _.y_scale_fn,
        Rect(0.0, 10.0, 5.0, 5.0))

    # Has x_origin: use it
    test_draw_2 = _.passes_1(
        2,
        _.x_scale_fn,
        _.y_scale_fn,
        Rect(2.0, 10.0, 3.0, 5.0))


    
                
                
            
if __name__ == '__main__':
    unittest.main()
