from rayon.scales import (
    LogScale, 
    CountingLogScale, 
    RayonScaleException,
    LogGradientScale,
    CountingLogGradientScale,
)
import unittest

class t_LogScale(unittest.TestCase):
    def test_attach_1(self):
        s = LogScale()
        s.attach([1,2,3], None)
        self.assertEqual(1, s.get_input_min())
        self.assertEqual(3, s.get_input_max())
            
    def test_attach_checks_input_1(self):
        s = LogScale()
        self.assertRaises(RayonScaleException,
                          lambda: s.attach(["1", "2", "3"], None))

    def test_attach_checks_input_2(self):
        s = LogScale()
        self.assertRaises(RayonScaleException,
                          lambda: s.attach([0], None))

class t_CountingLogScale(unittest.TestCase):

    # As part of the implementation, the input and output are
    # converted to floats when they are updateed. Some precision is
    # therefore lost, so exact comparistons aren't possible.
    #
    # This is a bug, and should probably be fixed in the future. The
    # right answer is to keep separate track of the input instead of
    # relying on the underlying LogScale.

    def test_attach_1(self):
        s = CountingLogScale()
        s.attach([1,2,3], None)
        self.assertAlmostEqual(1, s.get_input_min())
        self.assertAlmostEqual(3, s.get_input_max())
            
    def test_attach_2(self):
        # You can pass zeroes to CountingLogScale! All hail,
        # CountingLogScale!
        s = CountingLogScale()
        s.attach([0,1,2,3], None)
        self.assertAlmostEqual(0, s.get_input_min())
        self.assertAlmostEqual(3, s.get_input_max())
            
    def test_attach_checks_input_1(self):
        s = CountingLogScale()
        self.assertRaises(RayonScaleException,
                          lambda: s.attach(["1", "2", "3"], None))


class t_LogGradientScale(unittest.TestCase):
    def test_attach_1(self):
        s = LogGradientScale()
        s.attach([1,2,3], None)
        self.assertEqual(1, s.get_input_min())
        self.assertEqual(3, s.get_input_max())
            
    def test_attach_checks_input_1(self):
        s = LogGradientScale()
        self.assertRaises(RayonScaleException,
                          lambda: s.attach(["1", "2", "3"], None))

    def test_attach_checks_input_2(self):
        s = LogGradientScale()
        self.assertRaises(RayonScaleException,
                          lambda: s.attach([0], None))

class t_CountingLogGradientScale(unittest.TestCase):

    # Same input caveats apply as CountingLogGradientScale

    def test_attach_1(self):
        s = CountingLogGradientScale()
        s.attach([1,2,3], None)
        self.assertAlmostEqual(1, s.get_input_min())
        self.assertAlmostEqual(3, s.get_input_max())
            
    def test_attach_2(self):
        # You can pass zeroes to CountingLogGradientScale! All hail,
        # CountingLogGradientScale!
        s = CountingLogGradientScale()
        s.attach([0,1,2,3], None)
        self.assertAlmostEqual(0, s.get_input_min())
        self.assertAlmostEqual(3, s.get_input_max())
            
    def test_attach_checks_input_1(self):
        s = CountingLogGradientScale()
        self.assertRaises(RayonScaleException,
                          lambda: s.attach(["1", "2", "3"], None))

